(* colcom -- solve the colorful components problem
   Copyright (C) 2011, 2012  Falk Hüffner

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  *)

let search_tree_nodes = ref 0;;

type edge = int * int;;

type t = {
  g : Graph.t;
  epsilon : float;
  branching_paths : int list list;
  t_bridge_candidates : IntSet.t;
};;

let make g epsilon = {
  g = g;
  epsilon = epsilon;
  branching_paths = [];
  t_bridge_candidates = Graph.vertices g;
};;

let do_merge g u v =
  let g' = Graph.delete_vertex g v in
  let g' = Graph.modify_vertex_label (fun c_u -> IntSet.union c_u (Graph.vertex_label g v)) g' u in
  let g' = Graph.fold_neighbors
    (fun g' w w_vw ->
      if w = u then g'
      else if Graph.is_connected g u w
      then Graph.modify_edge_label (fun w_uw -> w_uw +. w_vw) g' u w
      else Graph.connect g' u w w_vw)
    g v g' in
  g'
;;

let merge g u v =
  let g' = do_merge g u v in
  let c_u = Graph.vertex_label g' u in
  let g', dk, red = Graph.fold_neighbors
    (fun (g', dk, red) v w_uv ->
      if IntSet.do_intersect c_u (Graph.vertex_label g' v)
      then Graph.disconnect g' u v, dk +. w_uv, (u, v) :: red
      else g', dk, red)
    g' u (g', 0.0, []) in
  g', dk,
  (fun del ->
    let del = red @ del in
    let del = List.fold_left
      (fun del (x, y) ->
	if x <> u && y <> u then (x, y) :: del
	else
	  let w = if x = u then y else x in	
	  let del = if Graph.is_connected g u w then (u, w) :: del else del in
	  let del = if Graph.is_connected g v w then (v, w) :: del else del in
	  del)
      [] del in
    del)
;;

let merge_cost g u v =
  let g' = do_merge g u v in
  let c_u = Graph.vertex_label g' u in
  Graph.fold_neighbors
    (fun dk v w_uv ->
      if IntSet.do_intersect c_u (Graph.vertex_label g' v)
      then dk +. w_uv
      else dk)
    g' u 0.0
;;
