(* colcom -- solve the colorful components problem
   Copyright (C) 2011, 2012  Falk Hüffner

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  *)

(** A Colorful Components problem instance.  *)

(** Statistics counter for the number of search tree nodes.  *)
val search_tree_nodes : int ref

(** The type of a Colorful Components problem instance.  *)
type t = {
  g : Graph.t;				(* graph *)
  epsilon : float;			(* minimum distance between weights *)
  branching_paths : int list list;	(* cached potential branching paths *)
  t_bridge_candidates : IntSet.t;	(* cached vertices for starting reduction search *)
}

(** [make g eps] returns a problem instance for graph [g] with
    minimum distance between weights [eps].  *)
val make : Graph.t -> float -> t

(** [merge g u v] returns a triple [(g', w, restore)], where [g'] is
    [g] with [u] and [v] merged, [w] is the cost of this, and
    [restore] is a function that from a solution for [g'] makes a
    solution for [g].  *)
val merge : Graph.t -> int ->  int ->
  Graph.t * float * ((int * int) list -> (int * int) list)

(** [merge_cost g u v] returns cost for merging [u] and [v] in [g].  *)
val merge_cost : Graph.t -> int ->  int -> float
