(* colcom -- solve the colorful components problem
   Copyright (C) 2011, 2012  Falk Hüffner

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  *)

exception Got_it of int * int

let pack i j =
  assert (i >= 0 && i < 1 lsl 29);
  assert (j >= 0 && j < 1 lsl 29);
  let i, j = if i < j then i, j else j, i in
    i lor (j lsl 30)
;;
let unpack x =
  x land ((1 lsl 30) - 1), x lsr 30
;;
let (@@) f g = fun x -> f (g x);;

let solve_merge g =
  let score g u v =
    let w_uv = Graph.edge_label g u v in
    let cut = IntMap.intersection
      (fun cut _ w1 w2 -> cut +. min w1 w2)
      (Graph.neighbors g u)
      (Graph.neighbors g v)
      (3.0 *. w_uv) in
    let merge_cost = Colcom.merge_cost g v u in
    let score = cut -. merge_cost in
    let score = -.score in
    score in
  let update g q u v = PSQueue.add q (pack u v) () (score g u v) in
  (*
  let verify_queue g q =
    PSQueue.fold (fun () e () sc ->
      let u, v = unpack e in
      if sc <> score g u v then Printf.eprintf "fail %d %d is %f should be %f\n%!" u v sc (score g u v);
      assert (sc = (score g u v))
    ) q () in
  *)
  let q = Graph.fold_edges (fun q u v _ -> update g q u v) g PSQueue.empty in
  let rec loop q g cost restore =
    if PSQueue.is_empty q then restore [], cost
    else
      let e, (), _ = PSQueue.top q in
      let q = PSQueue.pop q in
      let u, v = unpack e in
      let q = Graph.fold_neighbors (fun q w _ -> PSQueue.remove q (pack u w)) g u q in
      let q = Graph.fold_neighbors (fun q w _ -> PSQueue.remove q (pack v w)) g v q in
      let n1 = IntSet.union (Graph.neighbor_set g u) (Graph.neighbor_set g v) in
      let n2 = IntSet.fold (fun n2 w -> IntSet.union n2 (Graph.neighbor_set g w)) n1 n1 in
      let n2 = IntSet.remove n2 v in
      let g, dk, restore' = Colcom.merge g u v in
      let gn = Graph.subgraph g n2 in
      let q = Graph.fold_edges (fun q w x _ -> update g q w x) gn q in
      (* verify_queue g q; *)
      loop q g (cost +. dk) (restore @@ restore')
  in
  
  loop q g 0.0 (fun x -> x)
;;

(* version without caching  *)
let rec solve_merge' g =
  match
    Graph.fold_edges
      (fun (best, best_score) u v w_uv ->
	let n = IntSet.intersection (Graph.neighbor_set g u) (Graph.neighbor_set g v) in
	let cut =
	  IntSet.fold
	    (fun cut w -> cut +. min (Graph.edge_label g u w) (Graph.edge_label g v w)) n (3.0 *. w_uv) in
	let _, merge_cost, _ = Colcom.merge g v u in
	let score = cut -. merge_cost in
	if score > best_score
	then (u, v), score
	else best, best_score)
      g ((-1, -1), (-.infinity))
  with
    | (-1, -1), _ -> [], 0.0
    | (u, v), _ ->
      let g, dk, restore = Colcom.merge g v u in
      let del, cost = solve_merge g in
      restore del, cost +. dk
;;
