(* colcom -- solve the colorful components problem
   Copyright (C) 2011, 2012  Falk Hüffner

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  *)

(** Priority search queues with integer as keys, floats as
    priorities, and an arbitraty value stored per key.  *)

(** The type of the queue.  *)
type 'a t


(* The empty queue.  *)
val empty : 'a t

(* [add q k v p] adds to [q] the key [k] with value [v] and priority [p].
   O(log n) time.  *)
val add : 'a t -> int -> 'a -> float -> 'a t

(** Test whether the set is empty. O(1) time.  *)
val is_empty : 'a t -> bool

(** Returns the element with minimum priority.  O(log n) time.  *)
val top : 'a t -> int * 'a * float

(** Key lookup.  O(log n) time.  *)
val get : 'a t -> int -> 'a * float

(** Removes the element with minimum priority.  O(log n) time.  *)
val pop : 'a t -> 'a t

(** Removes an element by key. O(log n) time.  *)
val remove : 'a t -> int -> 'a t

(** [fold f q a] computes [(f iN xN pN ... (f i1 x1 p1 a)...)], where
    [i1 ... iN] are the keys of all bindings in [q] (in increasing
    order), and [x1 ... xN] and [p1 ... pN] are the associated data
    and priorities, respectively.  *)
val fold : ('b -> int -> 'a -> float -> 'b) -> 'a t -> 'b -> 'b
