(* colcom -- solve the colorful components problem
   Copyright (C) 2011  Falk Hüffner

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  *)

let verbose = ref false;;

let fold_n f n accu =
  let rec loop accu i =
    if i >= n
    then accu
    else loop (f accu i) (succ i)
  in
    loop accu 0
;;

let string_of_float f =
  let s = string_of_float f in
  if s.[String.length s - 1] = '.'
  then String.sub s 0 (String.length s - 1)
  else s
;;

let output_int channel x = Printf.fprintf channel "%d" x;;
let output_float channel x = Printf.fprintf channel "%s" (string_of_float x);;

let output_list p channel l =
  Printf.fprintf channel "[(%d) " (List.length l);
  ignore (List.fold_left
    (fun first x ->
       if not first then Printf.fprintf channel ", ";
       Printf.fprintf channel "%a" p x;
       false)
    true
    l);
  Printf.fprintf channel "]";
;;

let output_file channel filename =
  let cin = open_in_bin filename in
  let buflen = 8192 in
  let buf = String.create buflen in
  let rec loop () =
    let n = input cin buf 0 buflen in
    if n <> 0 then begin
      output channel buf 0 n;
      loop ();
    end
  in
  loop ();
  flush channel;
;;

let timer () =
  let stamp = Unix.times () in
    stamp.Unix.tms_utime +. stamp.Unix.tms_cutime
;;
