(* colcom -- solve the colorful components problem
   Copyright (C) 2011  Falk Hüffner

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  *)

(** General helper functions.  *)

(* Global verboseness option.  *)
val verbose : bool ref

(** [fold_n f n accu] calculates [f (... f (f accu 0) 1 ... (n-1))].  *)
val fold_n : ('a -> int -> 'a) -> int -> 'a -> 'a

(** Converts a float to a string without trailing zeros.  *)
val string_of_float : float -> string

(** [output_int c i] prints [i] on channel [c].  *)
val output_int : out_channel -> int -> unit

(** [output_float c f] prints [f] on channel [c].  *)
val output_float : out_channel -> float -> unit

(** [output_list p c l] prints a debug representation of list [l] to
    channel [c], using [p] to print values.  *)
val output_list : (out_channel -> 'a -> unit) -> out_channel -> 'a list -> unit

(** [output_file c filename] prints the contents of [filename] to
    channel [c].  *)
val output_file : out_channel -> string -> unit

(** Current CPU time.  *)
val timer : unit -> float
